package com.csc.library.websocket;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.websocket.ContainerProvider;
import javax.websocket.DeploymentException;
import javax.websocket.Session;

import com.csc.library.authen.WebSocketAuthen;
import com.csc.library.authen.WebSocketMyAuthen;
import com.csc.library.factory.PropertiesFactory;
import com.csc.library.factory.SessionFactory;
import com.csc.library.fix.StaticVariable;
import com.csc.library.properties.CscDevice;
import com.csc.library.properties.CscSession;
import com.csc.library.properties.DeviceProperties;
import com.csc.library.utilities.UProfile;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

public class ServerWebSocket {
	private static ServerWebSocket instance = null;
	
	private String serverName = "";
	private String urlGateWay = "";
	private Class classHandle = null;
	private Session sessionGateWay = null;
	private Map<String , CscSession> sessions = new HashMap<String , CscSession>();
	private WebSocketAuthen authen = new WebSocketMyAuthen();
	
	private ServerWebSocket(){
		//load config xml websocket server
		//load serverName,urlGateWay,classHandle
	}
	
	public static ServerWebSocket getInstance(){
		if(instance == null){
			instance = new ServerWebSocket();
			try {
				if(instance.classHandle != null && !instance.urlGateWay.equals("")){
					instance.sessionGateWay = ContainerProvider.getWebSocketContainer()
						.connectToServer(instance.classHandle , new URI(instance.urlGateWay));
				}
			} catch (DeploymentException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (URISyntaxException e) {
				e.printStackTrace();
			}
		}
		return instance;
	}
	
	public String getServerName(){
		return this.serverName;
	}
	
	public boolean registerSession(Session session , String jsonProp){
		try{
			/* check user login */
			Map<String,Object> prop = session.getUserProperties();
			CscDevice deviceProp = PropertiesFactory.getInstance().createDevice(jsonProp);
			UProfile upf =  authen.checkPermission(deviceProp);
			if(upf != null){
				prop.put(StaticVariable.UPROFILE , upf);
				prop.put(StaticVariable.DEVICE , deviceProp);
				CscSession ses = SessionFactory.getInstance().createSession(upf, session, deviceProp);
				this.addSession(deviceProp, ses);
				return true;
			}else{
				return false;
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return false;
	}
	
	public Map<String, CscSession> getSessions(){
		return Collections.unmodifiableMap(sessions);
	}
	
	public void addSession(CscDevice prop , CscSession session){
		sessions.put(prop.getUserName(), session);
	}
	
	public CscSession getCscSession(Session session){
		Map<String,Object> prop = session.getUserProperties();
		if( session != null && prop.containsKey(StaticVariable.DEVICE) 
				&& prop.containsKey(StaticVariable.UPROFILE)){
			CscDevice device = (CscDevice)prop.get(StaticVariable.DEVICE);
			if(sessions.containsKey(device.getUserName())){
				return sessions.get(device.getUserName());
			}
		}
		return null;
	}
}
